package cz.mendelu.dnaAnalyser.analyse.g4hunter.quadruplex;

import cz.mendelu.dnaAnalyser.utils.analyse.data.DataLocalisedMapper;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.cursor.Cursor;
import org.apache.ibatis.session.RowBounds;

import java.util.List;

public interface QuadruplexMapper extends DataLocalisedMapper<Quadruplex> {

    // language=SQL
    String TABLE = "CREATE TABLE QUADRUPLEX (" +
            "ID integer NOT NULL AUTO_INCREMENT, " +
            "POSITION integer, " +
            "LENGTH integer, " +
            "SCORE float, " +
            "SEQUENCE varchar, " +
            "PRIMARY KEY (ID))";

    // language=SQL
    @Select("SELECT count(*) FROM QUADRUPLEX")
    int countAll();

    // language=SQL
    @Select("SELECT * FROM QUADRUPLEX")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "sequence", column = "SEQUENCE")
    })
    List<Quadruplex> getAll(RowBounds rowBounds);

    // language=SQL
    @Select("SELECT * FROM QUADRUPLEX")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "sequence", column = "SEQUENCE")
    })
    Cursor<Quadruplex> getCursor();

    // language=SQL
    @Select("SELECT count(*) FROM QUADRUPLEX WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition}")
    int countAllBetweenPosition(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition);

    // language=SQL
    @Select("SELECT * FROM QUADRUPLEX")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "sequence", column = "SEQUENCE")
    })
    List<Quadruplex> getAllItems();

    // language=SQL
    @Select("SELECT * FROM QUADRUPLEX WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition} ORDER BY ${orderName} ASC")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "sequence", column = "SEQUENCE")
    })
    List<Quadruplex> getAllBetweenPositionOrderAsc(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition,
            @Param("orderName") String orderName,
            RowBounds rowBounds);

    // language=SQL
    @Select("SELECT * FROM QUADRUPLEX WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition} ORDER BY ${orderName} DESC")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "sequence", column = "SEQUENCE")
    })
    List<Quadruplex> getAllBetweenPositionOrderDesc(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition,
            @Param("orderName") String orderName,
            RowBounds rowBounds);

    // language=SQL
    @Select("SELECT * FROM QUADRUPLEX WHERE ID = #{id}")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "sequence", column = "SEQUENCE"),
    })
    Quadruplex getById(int id);

    // language=SQL
    @Update("UPDATE QUADRUPLEX SET POSITION = #{position}, LENGTH = #{length}, SCORE = #{score}, SEQUENCE = #{sequence} WHERE ID = #{id}")
    void update(Quadruplex quadruplex);

    // language=SQL
    @Delete("DELETE from QUADRUPLEX WHERE ID = #{id}")
    void delete(int id);

    // language=SQL
    @Insert("INSERT INTO QUADRUPLEX (ID, POSITION, LENGTH, SCORE, SEQUENCE) VALUES (#{id}, #{position}, #{length}, #{score}, #{sequence})")
    @Options(useGeneratedKeys = true, keyProperty = "id")
    void insert(Quadruplex quadruplex);

}
