package cz.mendelu.dnaAnalyser.jwt;

import cz.mendelu.dnaAnalyser.user.UserService;
import cz.mendelu.dnaAnalyser.utils.exception.NotFoundException;
import cz.mendelu.dnaAnalyser.user.User;
import cz.mendelu.dnaAnalyser.user.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.UUID;

@Service
public class JwtService {

    @Value("${security.jwt.header}")
    private String tokenHeader;

    @Autowired
    private AuthenticationManager authenticationManager;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private JwtTokenUtil jwtTokenUtil;

    @Autowired
    private UserService userService;

    public String createToken(String login, String password) {
        // Perform the security
        Authentication authentication = authenticationManager.authenticate(
                new UsernamePasswordAuthenticationToken(login, password)
        );
        SecurityContextHolder.getContext().setAuthentication(authentication);

        // Reload password post-security so we can generate token
        User user = userRepository.findByUsername(login)
                .orElseThrow(() -> new UsernameNotFoundException(String.format("No user found with username '%s'.", login)));

        // Return the token
        return jwtTokenUtil.generateToken(user);
    }

    public String createHostToken() {
        User host = User.host();
        host = userRepository.save(host);
        userService.createDefaultData(host);

        return jwtTokenUtil.generateToken(host);
    }

    public String refreshToken(String token) {
        UUID userId = jwtTokenUtil.getUserId(token);
        User user = userRepository
                .findById(userId)
                .orElseThrow(() -> new NotFoundException(User.class, userId));

        if (jwtTokenUtil.canTokenBeRefreshed(token, user.getLastPasswordResetDate())) {
            return jwtTokenUtil.refreshToken(token);
        } else {
            return null;
        }

    }


}
